@@ -630,7 +630,7 @@ typedef struct _Geis *Geis;
  * If no initialization arguments are passed, the parameter list consists of a
  * single NULL argument.
  */
-GEIS_API Geis geis_new(GeisString init_arg_name, ...);
+GEIS_API GEIS_VARARG Geis geis_new(GeisString init_arg_name, ...);
 
 /**
  * Cleans up an instance of the GEIS v2.0 API.
@@ -748,6 +748,62 @@ GEIS_API GeisString geis_error_message(Geis geis, GeisSize index);
  */
 #define GEIS_CONFIG_SEND_SYNCHRONOS_EVENTS "com.canonical.oif.events.synchronous"
 
+/**
+ * @def GEIS_CONFIG_DRAG_THRESHOLD
+ * Movement threshold for recognizing a DRAG gesture (in meters).  Value type
+ * GeisFloat.
+ */
+#define GEIS_CONFIG_DRAG_THRESHOLD "com.canonical.oif.drag.threshold"
+
+/**
+ * @def GEIS_CONFIG_DRAG_TIMEOUT
+ * Timeout for recognizing a DRAG gesture (in milliseconds).  Value type
+ * GeisInteger.
+ */
+#define GEIS_CONFIG_DRAG_TIMEOUT "com.canonical.oif.drag.timeout"
+
+/**
+ * @def GEIS_CONFIG_PINCH_THRESHOLD
+ * Movement threshold for recognizing a PINCH gesture (in meters).  Value type
+ * GeisFloat.
+ */
+#define GEIS_CONFIG_PINCH_THRESHOLD "com.canonical.oif.pinch.threshold"
+
+/**
+ * @def GEIS_CONFIG_PINCH_TIMEOUT
+ * Timeout for recognizing a PINCH gesture (in milliseconds).  Value type
+ * GeisInteger.
+ */
+#define GEIS_CONFIG_PINCH_TIMEOUT "com.canonical.oif.pinch.timeout"
+
+/**
+ * @def GEIS_CONFIG_ROTATE_THRESHOLD
+ * Movement threshold for recognizing a ROTATE gesture (in meters).  Value type
+ * GeisFloat.
+ */
+#define GEIS_CONFIG_ROTATE_THRESHOLD "com.canonical.oif.rotate.threshold"
+
+/**
+ * @def GEIS_CONFIG_ROTATE_TIMEOUT
+ * Timeout for recognizing a ROTATE gesture (in milliseconds).  Value type
+ * GeisInteger.
+ */
+#define GEIS_CONFIG_ROTATE_TIMEOUT "com.canonical.oif.rotate.timeout"
+
+/**
+ * @def GEIS_CONFIG_TAP_THRESHOLD
+ * Movement threshold for recognizing a TAP gesture (in meters).  Value type
+ * GeisFloat.
+ */
+#define GEIS_CONFIG_TAP_THRESHOLD "com.canonical.oif.tap.threshold"
+
+/**
+ * @def GEIS_CONFIG_TAP_TIMEOUT
+ * Timeout for recognizing a TAP gesture (in milliseconds).  Value type
+ * GeisInteger.
+ */
+#define GEIS_CONFIG_TAP_TIMEOUT "com.canonical.oif.tap.timeout"
+
 /* @} */
 
 /**
@@ -1600,9 +1656,9 @@ typedef struct _GeisRegion *GeisRegion;
  *
  * @returns a newly created region, or NULL on failure.
  */
-GEIS_API GeisRegion geis_region_new(Geis       geis,
-                                    GeisString name,
-                                    GeisString init_arg_name, ...);
+GEIS_API GEIS_VARARG GeisRegion geis_region_new(Geis       geis,
+                                                GeisString name,
+                                                GeisString init_arg_name, ...);
 
 /**
  * Destroys a GEIS v2.0 region.
@@ -1730,10 +1786,17 @@ GEIS_API GeisString geis_filter_name(GeisFilter filter);
  *   GEIS_GESTURE_ATTRIBUTE_TOUCHES, GEIS_FILTER_OP_EQ, 3,
  *   NULL);
  * @endcode
+ *
+ * Term descriptions are usually ANDed together, so that specifying a class name
+ * and a number of touches will filter only for gestures that have both
+ * characteristics. But if you specify several class names (e.g. drag and pinch),
+ * those classes are ORed together instead. So you can receive events from a
+ * gesture that belongs to either drag, drag&pinch or only pinch classes.
+ * 
  */
-GEIS_API GeisStatus geis_filter_add_term(GeisFilter         filter,
-                                         GeisFilterFacility facility,
-                                         ...);
+GEIS_API GEIS_VARARG GeisStatus geis_filter_add_term(GeisFilter         filter,
+                                                     GeisFilterFacility facility,
+                                                     ...);
 
 /* @} */
 
@@ -1847,9 +1910,10 @@ GEIS_API GeisInteger geis_subscription_id(GeisSubscription subscription);
  * @param[in] subscription  The subscription.
  * @param[in] filter        The filter to be added to the subscription.
  *
- * Thye effect of filters are ANDed together so that, for example, a
- * subscription that has a filter for 3-finger gestures and drag gestures will
- * produce events only for 3-finger drag gestures.
+ * The effect of filters are ORed together so that, for example, a
+ * subscription that has a filter for 3-finger drag gestures and another for
+ * 2-finger pinch gestures will produce events for both 3-finger drag gestures
+ * and 2-finger pinch gestures.
  *
  * The default is no filters:  that is, all possible gesture events will be
  * reported.
@@ -1884,6 +1948,45 @@ GEIS_API GeisFilter geis_subscription_filter_by_name(GeisSubscription sub,
 GEIS_API GeisStatus geis_subscription_remove_filter(GeisSubscription subscription,
                                                     GeisFilter       filter);
 
+
+/**
+ * Gets a subscription-level configuration item.
+ *
+ * @param[in]  subscription      The subscription from which the configuration
+ *                               item will be retrieved.
+ * @param[in]  config_item_name  The name of the configuration item.
+ * @param[out] config_item_value A pointer to an appropriate variable to hold
+ *                               the retrieved config item value.
+ *
+ * Not all back ends support all configuration items.
+ *
+ * @retval GEIS_STATUS_BAD_ARGUMENT   an invalid argument value was passed
+ * @retval GEIS_STATUS_NOT_SUPPORTED  the configuration value is not supported
+ * @retval GEIS_STATUS_SUCCESS        normal successful completion
+ */
+GEIS_API GeisStatus geis_subscription_get_configuration(GeisSubscription subscription,
+                                                        GeisString       config_item_name,
+                                                        GeisPointer      config_item_value);
+
+/**
+ * Sets a subscription-level configuration item.
+ *
+ * @param[in] subscription      The subscription from which the configuration
+ *                              item will be retrieved.
+ * @param[in] config_item_name  The name of the configuration item.
+ * @param[in] config_item_value A pointer to an appropriate variable holding
+ *                              the config item value.
+ *
+ * Not all back ends support all configuration items.
+ *
+ * @retval GEIS_STATUS_BAD_ARGUMENT   an invalid argument value was passed
+ * @retval GEIS_STATUS_NOT_SUPPORTED  the configuration value is not supported
+ * @retval GEIS_STATUS_SUCCESS        normal successful completion
+ */
+GEIS_API GeisStatus geis_subscription_set_configuration(GeisSubscription subscription,
+                                                        GeisString       config_item_name,
+                                                        GeisPointer      config_item_value);
+
 /* @} */
 
 /**
@@ -1952,9 +2055,27 @@ typedef struct _GeisFrame    *GeisFrame;
  *
  * @def GEIS_EVENT_ATTRIBUTE_TOUCHSET
  * The event attribute containing a pointer to a GeisTouchSet.
+ *
+ * @def GEIS_EVENT_ATTRIBUTE_CONSTRUCTION_FINISHED
+ * Event attribute containing a boolean.
+ * This property allows the client to determine if all the possible gestures
+ * from the set of touches in this event have already been presented. When
+ * this value is true, the client will have received all the information needed
+ * to make a gesture accept and reject decision based on potentially
+ * overlapping gestures. An example is when both one and two touch gestures are
+ * subscribed on the same window with the same gesture classes and thresholds.
+ * When this property is true for one touch gesture events, the client can be
+ * sure there are no other touches unless a two touch gesture event has already
+ * been sent.
+ * Another example is when you subscribe for three touches Touch and four
+ * touches Drag. As soon as a third finger is detected a three touches Touch
+ * gesture will begin, but you cannot be sure a fourth finger isn't coming
+ * right after (that can eventually cause a four touches Drag) until this
+ * property is true.
  */
 #define GEIS_EVENT_ATTRIBUTE_GROUPSET  "group set"
 #define GEIS_EVENT_ATTRIBUTE_TOUCHSET  "touch set"
+#define GEIS_EVENT_ATTRIBUTE_CONSTRUCTION_FINISHED  "construction finished"
 
 /* @} */
 
