@@ -107,6 +107,28 @@ struct GeisGrailBackend
 };
 
 
+/**
+ * A table of supported grail cofiguration properties and their GEIS
+ * equivalents.
+ */
+static const struct GeisGrailConfigProperties
+{
+  GeisString             geis_config_name;
+  GeisAttrType           geis_value_type;
+  UGSubscriptionProperty grail_property_name;
+} _grail_be_config_properties[] = {
+  { GEIS_CONFIG_DRAG_TIMEOUT,     GEIS_ATTR_TYPE_INTEGER, UGSubscriptionPropertyDragTimeout },
+  { GEIS_CONFIG_DRAG_THRESHOLD,   GEIS_ATTR_TYPE_FLOAT,   UGSubscriptionPropertyDragThreshold },
+  { GEIS_CONFIG_PINCH_TIMEOUT,    GEIS_ATTR_TYPE_INTEGER, UGSubscriptionPropertyPinchTimeout },
+  { GEIS_CONFIG_PINCH_THRESHOLD,  GEIS_ATTR_TYPE_FLOAT,   UGSubscriptionPropertyPinchThreshold },
+  { GEIS_CONFIG_ROTATE_TIMEOUT,   GEIS_ATTR_TYPE_INTEGER, UGSubscriptionPropertyRotateTimeout },
+  { GEIS_CONFIG_ROTATE_THRESHOLD, GEIS_ATTR_TYPE_FLOAT,   UGSubscriptionPropertyRotateThreshold },
+  { GEIS_CONFIG_TAP_TIMEOUT,      GEIS_ATTR_TYPE_INTEGER, UGSubscriptionPropertyTapTimeout },
+  { GEIS_CONFIG_TAP_THRESHOLD,    GEIS_ATTR_TYPE_FLOAT,   UGSubscriptionPropertyTapThreshold },
+  { NULL, 0, 0 }
+};
+
+
 static GeisStatus
 _grail_be_activate_for_device(GeisGrailBackend gbe,
                               GeisFilter       filter,
@@ -468,6 +490,39 @@ _grail_be_set_x11_timeout(GeisGrailBackend gbe)
     geis_grail_xsync_set_timeout(gbe->xsync, timeout);
 }
 
+/**
+ * Creates a GEIS_EVENT_ATTRIBUTE_CONSTRUCTION_FINISHED corresponding to the
+ * "construction finished" property of the given slice.
+ */
+static GeisAttr
+_grail_be_create_construction_attr(UGSlice slice)
+{
+  int slice_construction_finished = 0;
+
+  UGStatus status = grail_slice_get_property(slice,
+      UGSlicePropertyConstructionFinished,
+      &slice_construction_finished);
+  if (status != UGStatusSuccess)
+  {
+    geis_error("could not retrieve \"construction finished\" property from "
+               "grail slice");
+    return 0;
+  }
+
+  GeisBoolean attr_value;
+  if (slice_construction_finished)
+    attr_value = GEIS_TRUE;
+  else
+    attr_value = GEIS_FALSE;
+
+  GeisAttr construction_attr = geis_attr_new(
+      GEIS_EVENT_ATTRIBUTE_CONSTRUCTION_FINISHED,
+      GEIS_ATTR_TYPE_BOOLEAN,
+      &attr_value);
+
+  return construction_attr;
+}
+
 static void
 _grail_be_grail_fd_callback(int                             fd GEIS_UNUSED,
                             GeisBackendMultiplexorActivity  ev GEIS_UNUSED,
@@ -710,8 +765,12 @@ _grail_be_grail_fd_callback(int                             fd GEIS_UNUSED,
                                       slice, slice_state,
                                       time_delta,
                                       frame);
+
+      GeisAttr construction_attr = _grail_be_create_construction_attr(slice);
+
       geis_event_add_attr(geis_event, group_attr);
       geis_event_add_attr(geis_event, touch_attr);
+      geis_event_add_attr(geis_event, construction_attr);
       geis_post_event(gbe->geis, geis_event);
 
       /* Destroy any saved state after the a gesture has ended. */
@@ -1264,23 +1323,23 @@ _geis_grail_report_classes(GeisGrailBackend gbe)
                       / sizeof(struct GeisFilterableAttribute);
 
   gbe->drag_class = geis_gesture_class_new(GEIS_GESTURE_DRAG,
-                                           GEIS_GESTURE_PRIMITIVE_DRAG);
+                                           UGGestureTypeDrag);
   geis_register_gesture_class(gbe->geis, gbe->drag_class, attr_count, attrs);
 
   gbe->pinch_class = geis_gesture_class_new(GEIS_GESTURE_PINCH,
-                                            GEIS_GESTURE_PRIMITIVE_PINCH);
+                                            UGGestureTypePinch);
   geis_register_gesture_class(gbe->geis, gbe->pinch_class, attr_count, attrs);
 
   gbe->rotate_class = geis_gesture_class_new(GEIS_GESTURE_ROTATE,
-                                             GEIS_GESTURE_PRIMITIVE_ROTATE);
+                                             UGGestureTypeRotate);
   geis_register_gesture_class(gbe->geis, gbe->rotate_class, attr_count, attrs);
 
   gbe->tap_class = geis_gesture_class_new(GEIS_GESTURE_TAP,
-                                          GEIS_GESTURE_PRIMITIVE_TAP);
+                                          UGGestureTypeTap);
   geis_register_gesture_class(gbe->geis, gbe->tap_class, attr_count, attrs);
 
   gbe->touch_class = geis_gesture_class_new(GEIS_GESTURE_TOUCH,
-                                            GEIS_GESTURE_PRIMITIVE_TOUCH);
+                                            UGGestureTypeTouch);
   geis_register_gesture_class(gbe->geis, gbe->touch_class, attr_count, attrs);
 }
 
@@ -1634,12 +1693,169 @@ _grail_be_reject_gesture(GeisBackend   be,
   return status;
 }
 
+
+/**
+ * Sets a given property from the first ugsub in a GEIS subscription.
+ *
+ * If there is more than one grail subscription mapped to a geis subscription,
+ * only the property value from the first is retrieved assuming all the mapped
+ * subscriptions have the same property value setting.
+ */
+static GeisStatus
+_grail_be_get_ugsub_property(GeisSubscription       geis_subscription,
+                             UGSubscriptionProperty grail_property,
+                             GeisPointer            grail_value)
+{
+  GeisStatus retval = GEIS_STATUS_UNKNOWN_ERROR;
+  GeisUGSubscriptionStore ugstore = geis_subscription_pdata(geis_subscription);
+  if (ugstore)
+  {
+    for (GeisSize i = 0; i < geis_ugsubscription_count(ugstore); ++i)
+    {
+      UGSubscription ugsub = geis_ugsubscription_get_ugsubscription_at(ugstore,
+                                                                       i);
+      if (UGStatusSuccess == grail_subscription_get_property(ugsub,
+                                                             grail_property,
+                                                             grail_value))
+      {
+        retval = GEIS_STATUS_SUCCESS;
+        break;
+      }
+    }
+  }
+  return retval;
+}
+
+
+/**
+ * Sets a given grail property to a specified value.
+ */
+static GeisStatus
+_grail_be_set_ugsub_property(GeisSubscription       geis_subscription,
+                             UGSubscriptionProperty grail_property,
+                             GeisPointer            grail_value)
+{
+  GeisStatus retval = GEIS_STATUS_UNKNOWN_ERROR;
+  GeisUGSubscriptionStore ugstore = geis_subscription_pdata(geis_subscription);
+  if (ugstore)
+  {
+    for (GeisSize i = 0; i < geis_ugsubscription_count(ugstore); ++i)
+    {
+      UGSubscription ugsub = geis_ugsubscription_get_ugsubscription_at(ugstore,
+                                                                       i);
+      if (UGStatusSuccess == grail_subscription_set_property(ugsub,
+                                                             grail_property,
+                                                             grail_value))
+      {
+        retval = GEIS_STATUS_SUCCESS;
+      }
+    }
+  }
+  return retval;
+}
+
+
+/*
+ * Dispatches the get-configuration call.
+ */
+static GeisStatus
+_grail_be_get_configuration(GeisBackend      be GEIS_UNUSED,
+                            GeisSubscription subscription,
+                            GeisString       item_name,
+                            GeisPointer      item_value)
+{
+  GeisStatus retval = GEIS_STATUS_NOT_SUPPORTED;
+  for (int i = 0; _grail_be_config_properties[i].geis_config_name; ++i)
+  {
+    if (0 == strcmp(item_name, _grail_be_config_properties[i].geis_config_name))
+    {
+      switch (_grail_be_config_properties[i].geis_value_type)
+      {
+        case GEIS_ATTR_TYPE_INTEGER:
+        {
+          uint64_t value;
+          retval = _grail_be_get_ugsub_property(subscription,
+                            _grail_be_config_properties[i].grail_property_name,
+                            &value);
+          *(GeisInteger*)item_value = value;
+          break;
+        }
+        case GEIS_ATTR_TYPE_FLOAT:
+        {
+          float value;
+          retval = _grail_be_get_ugsub_property(subscription,
+                            _grail_be_config_properties[i].grail_property_name,
+                            &value);
+          *(GeisFloat*)item_value = value;
+          break;
+        }
+        default:
+          geis_error("configuration value of unknown type requested");
+          retval = GEIS_STATUS_UNKNOWN_ERROR;
+          break;
+      }
+      break;
+    }
+  }
+
+  return retval;
+}
+
+
+/*
+ * Dispatches the set-configuration call.
+ */
+static GeisStatus
+_grail_be_set_configuration(GeisBackend      be GEIS_UNUSED,
+                            GeisSubscription subscription,
+                            GeisString       item_name,
+                            GeisPointer      item_value)
+{
+  GeisStatus retval = GEIS_STATUS_NOT_SUPPORTED;
+
+  for (int i = 0; _grail_be_config_properties[i].geis_config_name; ++i)
+  {
+    if (0 == strcmp(item_name, _grail_be_config_properties[i].geis_config_name))
+    {
+      switch (_grail_be_config_properties[i].geis_value_type)
+      {
+        case GEIS_ATTR_TYPE_INTEGER:
+        {
+          uint64_t value = *(GeisInteger *)item_value;
+          retval = _grail_be_set_ugsub_property(subscription,
+                            _grail_be_config_properties[i].grail_property_name,
+                            &value);
+          break;
+        }
+        case GEIS_ATTR_TYPE_FLOAT:
+        {
+          float value = *(GeisFloat *)item_value;
+          retval = _grail_be_set_ugsub_property(subscription,
+                            _grail_be_config_properties[i].grail_property_name,
+                            &value);
+          break;
+        }
+        default:
+          geis_error("configuration value of unknown type requested");
+          retval = GEIS_STATUS_UNKNOWN_ERROR;
+          break;
+      }
+      break;
+    }
+  }
+
+  return retval;
+}
+
+
 static struct GeisBackendVtable gbe_vtbl = {
   _geis_grail_backend_construct,
   _geis_grail_backend_finalize,
   geis_grail_token_new,
   _grail_be_accept_gesture,
   _grail_be_reject_gesture,
+  _grail_be_get_configuration,
+  _grail_be_set_configuration,
 };
 
 
