@@ -898,11 +898,49 @@ geis_get_configuration(Geis        geis,
     *(GeisBoolean*)configuration_item_value = geis->ignore_device_events;
     status = GEIS_STATUS_SUCCESS; 
   }
+  else
+  {
+    status = geis_get_sub_configuration(geis,
+                                        NULL,
+                                        configuration_item_name,
+                                        configuration_item_value);
+  }
 
   return status;
 }
 
 
+/*
+ * Gets a feature configuration value from the current back end.
+ */
+GeisStatus
+geis_get_sub_configuration(Geis              geis, 
+                           GeisSubscription  sub,
+                           GeisString        item_name,
+                           void             *item_value)
+{
+  return geis_backend_get_configuration(geis->backend,
+                                        sub,
+                                        item_name,
+                                        item_value);
+}
+
+
+/*
+ * gets a feature configuration value in the current back end.
+ */
+GeisStatus
+geis_set_sub_configuration(Geis              geis,
+                           GeisSubscription  sub,
+                           GeisString        item_name, 
+                           void             *item_value)
+{
+  return geis_backend_set_configuration(geis->backend,
+                                        sub,
+                                        item_name,
+                                        item_value);
+}
+
 /*
  * Sets a named configuration item.
  */
@@ -926,6 +964,13 @@ geis_set_configuration(Geis        geis,
     geis->ignore_device_events = *(GeisBoolean*)configuration_item_value;
     status = GEIS_STATUS_SUCCESS; 
   }
+  else
+  {
+    status = geis_set_sub_configuration(geis,
+                                        NULL,
+                                        configuration_item_name,
+                                        configuration_item_value);
+  }
   return status;
 }
 
